package de.hdm_stuttgart.mi.sda2.usermanage.rdbms;

import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.Collection;


import org.apache.log4j.Logger;

import de.hdm_stuttgart.mi.sda2.usermanage.Cfg;
import de.hdm_stuttgart.mi.sda2.usermanage.parse.Group;

public class RdbmsHandler {
   
   private static Logger log = Logger
         .getLogger(RdbmsHandler.class);


   private final String insertGroupSql = "INSERT INTO SysGroup (gid, gidNumber) VALUES (? , ?)";

   private PreparedStatement insertGroupStmt;

   static {
      try {
         Class.forName(Cfg.get("RdbmsHandler.driverClassName"));
      } catch (ClassNotFoundException e) {
         System.err.println("Unable to register JDBC driver '"
               + Cfg.get("RdbmsHandler.jdbcUrl") + "'");
         System.exit(1);
      }
   }

   Connection conn = null;

   public void connect() throws SQLException {
      if (null != conn) {
         conn.rollback();
         conn.close();
      }
      conn = DriverManager.getConnection(Cfg.get("RdbmsHandler.jdbcUrl"),
            Cfg.get("RdbmsHandler.user"), Cfg.get("RdbmsHandler.password"));
      conn.setAutoCommit(false);
      insertGroupStmt = conn.prepareStatement(insertGroupSql, Statement.RETURN_GENERATED_KEYS);
   }

   public void addGroups(final Collection<Group> groups) {
      try {
         for (final Group g : groups) {
            insertGroupStmt.setString(1, g.gid);
            insertGroupStmt.setInt(2, g.gidNumber);
            insertGroupStmt.executeUpdate();
            addGeneratedId(insertGroupStmt, g);
         }
         conn.commit();
      } catch (SQLException e) {
         e.printStackTrace();
      }
   }
   static void addGeneratedId(final PreparedStatement stmt, final Group group) 
      throws SQLException {
      final ResultSet rs = stmt.getGeneratedKeys();
      if (rs.next()) {
         final int ret = rs.getInt(1);
         log.info("Generated id=" + ret);
         group.setDatabaseId(ret);
      } else {
         log.fatal("No id found for group '" + group.gid + "', exiting!");
         System.exit(1);
      }
   }
}
